#!/usr/bin/python3

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import json
import logging
import os
import requests
import socket
import uuid

from ansible.plugins.callback import CallbackBase

class CallbackModule(CallbackBase):
    """
    ansible alerta callback plugin
    ansible.cfg:
        callback_plugins   = <path_to_callback_plugins_folder>
        callback_whitelist = alerta
    and put the plugin in <path_to_callback_plugins_folder>

    This plugin makes use of the following required environment variables:
        ALERTA_COMPANY    : The inventory
        ALERTA_WEBHOOK    : The Alerta webhook
        ALERTA_APIKEY     : The Alerta API key
        ALERTA_GITLAB_PID : The project ID in GitLab
    """
    CALLBACK_VERSION = 2.0
    CALLBACK_TYPE = 'aggregate'
    CALLBACK_NAME = 'alerta'
    CALLBACK_NEEDS_WHITELIST = True

    playbook = ''

    def __init__(self):
        super(CallbackModule, self).__init__()

        logging.basicConfig(level=logging.FATAL)
        self.logger = logging.getLogger('alerta.ansible')
        self.logger.setLevel(logging.FATAL)
        self.hostname = socket.gethostname()
        self.company = os.getenv('ALERTA_COMPANY', '')
        self.pid = os.getenv('ALERTA_GITLAB_PID', '')
        self.session = str(uuid.uuid1())
        self.playbook = ''

    def _log(self, severity, task=None, result=None):
        # ignore_errors
        if self.logger.isEnabledFor(severity):
            values = {
                'host': self.hostname,
                'environment': self.company,
                'resource': self.session,
                'event': 'Exception',
                'tags': ['environment:%s' % self.company],
                'group': 'Application',
                'severity': 'critical',
                'service': ['www.example.com'],
                'text': '%s - %s' % (self.playbook, task),
                'value': '',
                'rawData': '',
            }
            if self.pid != '':
                values['attributes'] = {"gitlab_project_id": "%s" % self.pid}
            if result is not None:
                try:
                    values['rawData'] = json.dumps(result._result)
                except TypeError:
                    try:
                        values['rawData'] = json.dumps(result._result, skipkeys=True)
                    except TypeError:
                        values['rawData'] = 'Can not dump result data'
            headers = {
                'Authorization': 'Key %s' % os.getenv('ALERTA_APIKEY', ''),
                'Content-type': 'application/json'
            }
            requests.post('%s/alert' % os.getenv('ALERTA_WEBHOOK', ''), json=values, headers=headers)

    ##### PLAYBOOK callbacks #####

    def v2_playbook_on_start(self, playbook):
        self.playbook = playbook._file_name

    ##### RUNNER callbacks #####

    def v2_runner_on_ok(self, result, **kwargs):
        task_name = str(result._task).replace('TASK: ','')
        if 'failed' in result._result.keys() and result._result['failed']:
            severity = logging.FATAL
        elif 'changed' in result._result.keys() and result._result['changed']:
            severity = logging.INFO
        else:
            severity = logging.DEBUG
        self._log(severity, task=task_name, result=result)

    def v2_runner_on_failed(self, result, ignore_errors=False):
        if not ignore_errors:
            result._result['failed'] = True
            self.v2_runner_on_ok(result)

    def v2_runner_on_unreachable(self, result):
        self._log(logging.FATAL, result=result)

    def v2_runner_on_async_ok(self, result):
        self.v2_runner_on_ok(result)

    def v2_runner_on_async_failed(self, result):
        self.v2_runner_on_failed(result)

    def v2_runner_item_on_ok(self, result):
        self.v2_runner_on_ok(result)

    def v2_runner_item_on_failed(self, result):
        self.v2_runner_on_failed(result)
